@@ -217,7 +217,7 @@ static void fs_close(){
 }
 
 ///  readbitmap - read bitmap
-extern void readbitmap(char* device, image_head image_hdr, char* bitmap, int pui)
+extern void readbitmap(char* device, image_head image_hdr, unsigned long* bitmap, int pui)
 {
     unsigned char	*ntfs_bitmap;
     unsigned long long	current_block, used_block, free_block, pos;
@@ -260,10 +260,10 @@ extern void readbitmap(char* device, image_head image_hdr, char* bitmap, int pui
         if (bit == -1){                                     // Return -1 on error
 	    log_mesg(0, 1, 1, fs_opt.debug, "%s: check bitmap error\n", __FILE__); 
 	}else if(bit == 1){				    // The value of the bit (0 or 1)
-            bitmap[current_block] = 1;
+            pc_set_bit(current_block, bitmap);
             used_block++;
         } else {
-            bitmap[current_block] = 0;
+            pc_clear_bit(current_block, bitmap);
             free_block++;
         }
         /// update progress
